import pandas as pd
import numpy as np
import os
import matplotlib.pyplot as plt
import datetime

from cpdctrl.utility.file_io import get_next_filename, sanitize_filename

class DataCollector:
    def __init__(self, 
                 data_name: str, 
                 data_path: str,
                 header: dict[str, str]={},
                 dirname: str|None=None,
                 dir_exists_is_ok=False,
                 ):
        self.data = []
        self.name = data_name
        self.header = header
        self.path = os.path.abspath(os.path.expanduser(data_path))
        if dirname is None:
            self.dirname = sanitize_filename(datetime.datetime.now(datetime.UTC).strftime("%Y-%m-%d_%H-%M") + "_" + self.name)
        else:
            self.dirname = sanitize_filename(dirname)
        self.dirpath = os.path.join(self.path, self.dirname)

        if os.path.exists(self.dirpath):
            if not dir_exists_is_ok:
                raise Exception(f"Directory '{self.dirname}' already exists. Provide a different directory or pass `dir_exists_is_ok=True` to ignore this")
        else:
            os.makedirs(self.dirpath)
        self.flushed = False
    
    def _get_filename(self):
       return sanitize_filename(get_next_filename(self.name, self.dirpath, digits=5))

    def flush(self, verbose=False):
        # TODO check if dir still exists
        filename = self._get_filename() + ".csv"
        filepath = os.path.join(self.dirpath, filename)
        if verbose: print(f"Flushing data to {filepath}")
        self.to_dataframe().to_csv(filepath, sep=",", index=False, header=True)
        self.data = []
        self.flushed = True
        
    def finalize(self):
        if self.flushed:
            raise NotImplementedError()
        
    def clear(self):
        self.data = []

    def add_data(self, i, t, v, l):
        self.data.append((i, t, v, l))
    
    def to_dataframe(self):
        df = pd.DataFrame(self.data, columns=["idx", "t [s]", "V [V]", "LED [%]"])
        df.meta = str(self.header)
        return df
    
    def save_csv(self):
        self.to_dataframe().to_csv(os.path.join(self.path, self.name + ".csv"), index=False, header=True)
    
        

# deprecated
# def buffer2dataframe(buffer):
#     df = pd.DataFrame(buffer)
#     df.colums = ["Time [s]", "Voltage [V]"]
#     return df
# OLD STUFF
def buffers2dataframe(ibuffer, vbuffer):
    """
    @param ibuffer : 2d - array: timestamps, current
    @param vbuffer : 2d - array: timestamps, voltage
    @returns DataFrame: timestamps, current, voltage
    """
    df = pd.DataFrame(np.vstack((ibuffer[:,0], ibuffer[:,1], vbuffer[:,1])).T)
    df.columns = ["Time [s]", "Current [A]", "Voltage [V]"]
    return df

def load_dataframe(p:str):
    """
    Load a dataframe from file.
    @param p : path of the file. If it has 'csv' extension, pandas.read_csv is used, pandas.read_pickle otherwise
    """
    if not os.path.isfile(p):
        print(f"ERROR: load_dataframe: File does not exist: {p}")
        return None
    if p.endswith(".csv"):
        df = pd.read_csv(p)
    else:
        df = pd.read_pickle(p)
    return df

def plot(data: str or pd.DataFrame or np.ndarray, title="", U=True, I=False):
    """
    Plot recorded data
    @param data: filepath, dataframe or numpy array
    """
    if type(data) == str:
        _data = load_dataframe(data).to_numpy()
    elif type(data) == pd.DataFrame:
        _data = data.to_numpy()
    else:
        _data = data
    print(_data[0])
    plt.ion()
    fig, ax = plt.subplots()
    ax.set_xlabel("t [s]")
    vax = ax
    iax = ax
    if U and I:
        iax = ax.twinx()
    if U:
        vax = ax
        vax.set_ylabel("U [V]")
        vax.plot(_data[:,0], _data[:,2], color="blue", label="voltage")
    if I:
        iax.set_ylabel("I [A]")
        iax.plot(_data[:,0], _data[:,1], color="orange", label="current")
    if U and I:
        plt.legend()
    return fig





